package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.TmstruckReceiptResultMapper;
import com.steerinfo.dil.mapper.UtilsMapper;
import com.steerinfo.dil.model.TmstruckReceiptResult;
import com.steerinfo.dil.service.ITmstruckReceiptResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * TmstruckReceiptResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-10 10:10
 * 类描述
 * 修订历史：
 * 日期：2021-09-10
 * 作者：generator
 * 参考：
 * 描述：TmstruckReceiptResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckReceiptResultService")
public class TmstruckReceiptResultServiceImpl implements ITmstruckReceiptResultService {

    @Autowired
    private TmstruckReceiptResultMapper tmstruckReceiptResultMapper;

    @Autowired
    private UtilsMapper utilsMapper;

    /**
     * 查询所有收货实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getReceiveResult(Map<String, Object> map) {
        Integer orderType = (Integer) map.get("orderTypee");
        switch (orderType){
            case 1:
                break;
            case 2:
                break;
            case 3:
                break;
            case 4:
                return tmstruckReceiptResultMapper.getSteelNzReceiptResult(map); //apiId: 266
            case 5:
            case 6:
            case 7:
            case 8:
            case 17:
            case 18:
                return tmstruckReceiptResultMapper.getCGReceiptResult(map); //apiId: 107
            case 9:
            case 10:
                return tmstruckReceiptResultMapper.getInwardReceipt(map); //apiId: 107
            case 11:

        }
        return null;
    }

    /**
     * 新增一条虚拟收货实绩
     * @param map {resultTotalId：总实绩ID}
     * @return
     */
    public int addReceiptResult(Map<String, Object> map){
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(map.get("resultTotalId"));
        //根据总实绩ID查询收货实绩查看是否已收货
        Integer receiptResult = tmstruckReceiptResultMapper.getReceiptResult(resultTotalId);
        if(receiptResult != null){
            return 0;
        }
        //新增完卸车实绩后新增一条收货实绩
        TmstruckReceiptResult tmstruckReceiptResult = new TmstruckReceiptResult();
        tmstruckReceiptResult.setId(tmstruckReceiptResultMapper.selectMaxId());//添加主键
        tmstruckReceiptResult.setResultTotalId(resultTotalId);//添加总实绩Id
        tmstruckReceiptResult.setStatus(new BigDecimal(0)); // 卸货完成后添加的为虚拟收货实绩
        return tmstruckReceiptResultMapper.insertSelective(tmstruckReceiptResult);
    }

    /**
     * 确认收货（修改收货实绩为真实收货实绩）
     * @param map {resultTotalId：总实绩ID}
     * @return
     */
    public int updateReceiptResult(Map<String, Object> map){
        TmstruckReceiptResult tmstruckReceiptResult = new TmstruckReceiptResult();
        tmstruckReceiptResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        tmstruckReceiptResult.setStatus(new BigDecimal(1));
        tmstruckReceiptResult.setInsertUsername("admin");
        tmstruckReceiptResult.setInsertTime(new Date());
        return tmstruckReceiptResultMapper.updateByTotalResultId(tmstruckReceiptResult);
    }

    /**
     * 新增钢材到异地库达州站驻港人员确定收货
     * @param map
     * @return
     */
    public int addSteelNzReceiptResult(Map<String, Object> map){
        String orderNumber = (String) map.get("orderNumber");
        //通过运输订单号获取总实绩ID
        BigDecimal resultTotalId = utilsMapper.getTotalIdByOrderNumber(orderNumber);
        //获取异地库ID
        BigDecimal warehouseId =  DataChange.dataToBigDecimal(map.get("warehouseId"));
        TmstruckReceiptResult tmstruckReceiptResult = new TmstruckReceiptResult();
        tmstruckReceiptResult.setResultId(tmstruckReceiptResultMapper.selectMaxId());
        tmstruckReceiptResult.setResultTotalId(resultTotalId);
        tmstruckReceiptResult.setStatus(new BigDecimal(1));
        tmstruckReceiptResult.setWarehouseId(warehouseId);
        tmstruckReceiptResult.setInsertTime(new Date());
        tmstruckReceiptResult.setInsertUsername("admin");
        return tmstruckReceiptResultMapper.insertSelective(tmstruckReceiptResult);
    }
}
